# pylint: disable=line-too-long
"""Defining some data used throughout the code."""

import os
from pathlib import Path
from typing import Dict, List

from ndmanager.SHA1.cendl31 import CENDL31_SHA1
from ndmanager.SHA1.cendl32 import CENDL32_SHA1
from ndmanager.SHA1.endfb8 import ENDFB8_SHA1
from ndmanager.SHA1.endfb71 import ENDFB71_SHA1
from ndmanager.SHA1.jeff33 import JEFF33_SHA1
from ndmanager.SHA1.jeff311 import JEFF311_SHA1
from ndmanager.SHA1.jendl5 import JENDL5_SHA1
from ndmanager.SHA1.tendl19 import TENDL19_SHA1
from ndmanager.SHA1.tendl23 import TENDL23_SHA1
from ndmanager.SHA1.test import TEST_SHA1

try:
    NDMANAGER_ENDF6 = Path(os.environ["NDMANAGER_ENDF6"]).absolute()
except KeyError:
    NDMANAGER_ENDF6 = None

try:
    NDMANAGER_HDF5 = Path(os.environ["NDMANAGER_HDF5"]).absolute()
except KeyError:
    NDMANAGER_HDF5 = None

try:
    NDMANAGER_CHAINS = Path(os.environ["NDMANAGER_CHAINS"]).absolute()
except KeyError:
    NDMANAGER_CHAINS = None

try:
    NDMANAGER_MODULEPATH = Path(os.environ["NDMANAGER_MODULEPATH"]).absolute()
except KeyError:
    NDMANAGER_MODULEPATH = None

USERAGENT = (
    '--user-agent="Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:26.0)'
    ' Gecko/20100101 Firefox/26.0"'
)

IAEA_ROOT = "https://www-nds.iaea.org/public/download-endf/"

ENDF6_LIBS: Dict[str, Dict[str, str | List[str]]] = {
    "jeff33": {
        "fancyname": "JEFF-3.3",
        "sublibraries": ["decay", "n", "nfpy", "sfpy", "tsl"],
        "source": r"https://www-nds.iaea.org/public/download-endf/JEFF-3.3",
        "info": """Version 3.3 of the Joint Evaluated Fission and Fusion"""
        """ (JEFF) library distributed by OECD's Nuclear Energy Agency (NEA)""",
        "homepage": "https://www.oecd-nea.org/dbforms/data/eva/evatapes/jeff_31/index-JEFF3.1.1.html",
        "index": [
            "1 NSUB=4      Materials:3852  Size:45Mb    Zipped:9Mb                       [DECAY] Radioactive Decay Data",
            "2 NSUB=5      Materials:3     Size:372Kb   Zipped:94Kb                      [S/FPY] Spontaneous Fission Product Yields",
            "3 NSUB=10     Materials:562   Size:2Gb     Zipped:451Mb    20.MeV-200.MeV   [N]     Incident-Neutron Data",
            "4 NSUB=11     Materials:19    Size:5Mb     Zipped:2Mb      1.MeV-14.MeV     [N/FPY] Neutron-Induced Fission Product Yields",
            "5 NSUB=12     Materials:20    Size:150Mb   Zipped:37Mb     5.eV-20.MeV      [TSL]   Thermal Neutron Scattering Data",
            "",
            "Total: Materials:4456  Size:2Gb     Zipped:497Mb",
        ],
    },
    "jeff311": {
        "fancyname": "JEFF-3.1.1",
        "sublibraries": ["decay", "n", "nfpy", "p", "sfpy", "tsl"],
        "source": r"https://www-nds.iaea.org/public/download-endf/JEFF-3.1.1",
        "info": """Version 3.1.1 of the Joint Evaluated Fission and Fusion"""
        """ (JEFF) library distributed by OECD's Nuclear Energy Agency (NEA)""",
        "homepage": "https://www.oecd-nea.org/dbdata/jeff/jeff33/",
        "index": [
            "1 NSUB=4      Materials:3852  Size:42Mb    Zipped:9Mb      [DECAY] Radioactive Decay Data",
            "2 NSUB=5      Materials:3     Size:375Kb   Zipped:96Kb     [S/FPY] Spontaneous Fission Product Yields",
            "3 NSUB=10     Materials:381   Size:246Mb   Zipped:55Mb     [N]     Incident-Neutron Data",
            "4 NSUB=11     Materials:19    Size:5Mb     Zipped:2Mb      [N/FPY] Neutron-Induced Fission Product Yields",
            "5 NSUB=12     Materials:9     Size:27Mb    Zipped:10Mb     [TSL]   Thermal Neutron Scattering Data",
            "6 NSUB=10010  Materials:26    Size:21Mb    Zipped:6Mb      [P]     Incident-Proton Data",
            "",
            "Total: Materials:4290  Size:338Mb   Zipped:79Mb ",
        ],
    },
    "jendl5": {
        "fancyname": "JENDL-5-Aug2023",
        "sublibraries": [
            "ard",
            "d",
            "decay",
            "e",
            "g",
            "he4",
            "n",
            "nfpy",
            "p",
            "photo",
            "sfpy",
            "tsl",
        ],
        "source": r"https://www-nds.iaea.org/public/download-endf/JENDL-5-Aug2023",
        "info": """Version 5 of the Japanese Evaluated Nuclear Data Library (JENDL)"""
        """library distributed by JAEA""",
        "homepage": "https://wwwndc.jaea.go.jp/jendl/j5/j5.html",
        "index": [
            "1 NSUB=0       Materials:2684  Size:6Gb     Zipped:2Gb      140.MeV-200.MeV  [G]     Photo-Nuclear Data",
            "2 NSUB=3       Materials:100   Size:87Mb    Zipped:31Mb     100.GeV          [PHOTO] Photo-Atomic Interaction Data",
            "3 NSUB=4       Materials:4071  Size:142Mb   Zipped:29Mb                      [DECAY] Radioactive Decay Data",
            "4 NSUB=5       Materials:10    Size:2Mb     Zipped:373Kb                     [S/FPY] Spontaneous Fission Product Yields",
            "5 NSUB=6       Materials:100   Size:9Mb     Zipped:2Mb      100.GeV          [ARD]   Atomic Relaxation Data",
            "6 NSUB=10      Materials:795   Size:16Gb    Zipped:5Gb      20.MeV-200.MeV   [N]     Incident-Neutron Data",
            "7 NSUB=11      Materials:31    Size:7Mb     Zipped:2Mb      0.eV-14.MeV      [N/FPY] Neutron-Induced Fission Product Yields",
            "8 NSUB=12      Materials:62    Size:2Gb     Zipped:385Mb    5.eV-10.eV       [TSL]   Thermal Neutron Scattering Data",
            "9 NSUB=113     Materials:100   Size:26Mb    Zipped:8Mb      100.GeV          [E]     Electro-Atomic Interaction Data",
            "10 NSUB=10010  Materials:239   Size:4Gb     Zipped:952Mb    200.MeV          [P]     Incident-Proton Data",
            "11 NSUB=10020  Materials:9     Size:252Mb   Zipped:68Mb     200.MeV          [D]     Incident-Deuteron Data",
            "12 NSUB=20040  Materials:18    Size:91Mb    Zipped:19Mb     15.MeV           [HE4]   Incident-Alpha data",
            "",
            "Total: Materials:8219  Size:27Gb    Zipped:8Gb",
        ],
    },
    "endfb71": {
        "fancyname": "ENDF-B-VII.1",
        "sublibraries": [
            "ard",
            "d",
            "decay",
            "e",
            "g",
            "he3",
            "n",
            "nfpy",
            "p",
            "photo",
            "sfpy",
            "std",
            "t",
            "tsl",
        ],
        "source": r"https://www-nds.iaea.org/public/download-endf/ENDF-B-VII.1",
        "info": """Version 7.1 of the ENDF-B data library distributed by the NNDC""",
        "homepage": "https://www.nndc.bnl.gov/endf-b7.1/",
        "index": [
            "1 NSUB=0       Materials:163   Size:222Mb   Zipped:57Mb     [G]     Photo-Nuclear Data",
            "2 NSUB=3       Materials:100   Size:23Mb    Zipped:8Mb      [PHOTO] Photo-Atomic Interaction Data",
            "3 NSUB=4       Materials:3821  Size:66Mb    Zipped:14Mb     [DECAY] Radioactive Decay Data",
            "4 NSUB=5       Materials:9     Size:2Mb     Zipped:297Kb    [S/FPY] Spontaneous Fission Product Yields",
            "5 NSUB=6       Materials:100   Size:9Mb     Zipped:2Mb      [ARD]   Atomic Relaxation Data",
            "6 NSUB=10      Materials:423   Size:811Mb   Zipped:217Mb    [N]     Incident-Neutron Data",
            "7 NSUB=11      Materials:31    Size:7Mb     Zipped:2Mb      [N/FPY] Neutron-Induced Fission Product Yields",
            "8 NSUB=12      Materials:21    Size:32Mb    Zipped:11Mb     [TSL]   Thermal Neutron Scattering Data",
            "9 NSUB=19      Materials:8     Size:886Kb   Zipped:228Kb    [Std]   Neutron Cross Section Standards",
            "10 NSUB=113    Materials:100   Size:24Mb    Zipped:8Mb      [E]     Electro-Atomic Interaction Data",
            "11 NSUB=10010  Materials:48    Size:53Mb    Zipped:14Mb     [P]     Incident-Proton Data",
            "12 NSUB=10020  Materials:5     Size:398Kb   Zipped:92Kb     [D]     Incident-Deuteron Data",
            "13 NSUB=10030  Materials:3     Size:665Kb   Zipped:148Kb    [T]     Incident-Triton Data",
            "14 NSUB=20030  Materials:2     Size:527Kb   Zipped:117Kb    [HE3]   Incident-He3 data",
            "",
            "Total: Materials:4834  Size:2Gb     Zipped:330Mb",
        ],
    },
    "endfb8": {
        "fancyname": "ENDF-B-VIII.0",
        "sublibraries": [
            "ard",
            "d",
            "decay",
            "e",
            "g",
            "he3",
            "he4",
            "n",
            "nfpy",
            "p",
            "photo",
            "sfpy",
            "std",
            "t",
            "tsl",
        ],
        "source": r"https://www-nds.iaea.org/public/download-endf/ENDF-B-VIII.0",
        "info": """Version 8.0 of the ENDF-B data library distributed by the NNDC""",
        "homepage": "https://www.nndc.bnl.gov/endf-b8.0/",
        "index": [
            "1 NSUB=0       Materials:163   Size:222Mb   Zipped:57Mb     20.MeV-150.MeV   [G]     Photo-Nuclear Data",
            "2 NSUB=3       Materials:100   Size:88Mb    Zipped:36Mb     100.GeV          [PHOTO] Photo-Atomic Interaction Data",
            "3 NSUB=4       Materials:3821  Size:66Mb    Zipped:14Mb                      [DECAY] Radioactive Decay Data",
            "4 NSUB=5       Materials:9     Size:2Mb     Zipped:297Kb                     [S/FPY] Spontaneous Fission Product Yields",
            "5 NSUB=6       Materials:100   Size:9Mb     Zipped:2Mb      100.GeV          [ARD]   Atomic Relaxation Data",
            "6 NSUB=10      Materials:557   Size:2Gb     Zipped:329Mb    20.MeV-200.MeV   [N]     Incident-Neutron Data",
            "7 NSUB=11      Materials:31    Size:7Mb     Zipped:2Mb      20.MeV           [N/FPY] Neutron-Induced Fission Product Yields",
            "8 NSUB=12      Materials:34    Size:230Mb   Zipped:65Mb     5.eV             [TSL]   Thermal Neutron Scattering Data",
            "9 NSUB=19      Materials:11    Size:65Mb    Zipped:21Mb     20.MeV-150.MeV   [Std]   Neutron Cross Section Standards",
            "10 NSUB=113    Materials:100   Size:27Mb    Zipped:8Mb      100.GeV          [E]     Electro-Atomic Interaction Data",
            "11 NSUB=10010  Materials:49    Size:54Mb    Zipped:14Mb     2.MeV-150.MeV    [P]     Incident-Proton Data",
            "12 NSUB=10020  Materials:5     Size:414Kb   Zipped:97Kb     1.MeV-30.MeV     [D]     Incident-Deuteron Data",
            "13 NSUB=10030  Materials:5     Size:736Kb   Zipped:164Kb    20.MeV           [T]     Incident-Triton Data",
            "14 NSUB=20030  Materials:3     Size:810Kb   Zipped:194Kb    20.MeV           [HE3]   Incident-He3 data",
            "15 NSUB=20040  Materials:1     Size:9Kb     Zipped:3Kb      20.MeV           [HE4]   Incident-Alpha data",
            "",
            "Total: Materials:4989  Size:2Gb     Zipped:546Mb",
        ],
    },
    "tendl19": {
        "fancyname": "TENDL-2019",
        "sublibraries": ["d", "g", "he3", "he4", "n", "p", "t"],
        "source": r"https://www-nds.iaea.org/public/download-endf/TENDL-2019",
        "info": "2019 release of the TENDL library distributed by the Paul Scherrer Institute (Switzerland).",
        "homepage": "https://tendl.web.psi.ch/tendl_2019/tendl2019.html",
        "index": [
            "1 NSUB=0      Materials:2809  Size:6Gb     Zipped:2Gb      30.MeV-200.MeV   [G]     Photo-Nuclear Data",
            "2 NSUB=10     Materials:2813  Size:12Gb    Zipped:3Gb      20.MeV-200.MeV   [N]     Incident-Neutron Data",
            "3 NSUB=10010  Materials:2812  Size:8Gb     Zipped:2Gb      2.MeV-200.MeV    [P]     Incident-Proton Data",
            "4 NSUB=10020  Materials:2810  Size:10Gb    Zipped:3Gb      1.MeV-200.MeV    [D]     Incident-Deuteron Data",
            "5 NSUB=10030  Materials:2805  Size:11Gb    Zipped:3Gb      20.MeV-200.MeV   [T]     Incident-Triton Data",
            "6 NSUB=20030  Materials:2807  Size:10Gb    Zipped:3Gb      20.MeV-200.MeV   [HE3]   Incident-He3 data",
            "7 NSUB=20040  Materials:2808  Size:6Gb     Zipped:2Gb      200.MeV          [HE4]   Incident-Alpha data",
            "",
            "Total: Materials:19664 Size:60Gb    Zipped:15Gb",
        ],
    },
    "tendl23": {
        "fancyname": "TENDL-2023",
        "sublibraries": ["d", "g", "he3", "he4", "n", "p", "t"],
        "source": r"https://www-nds.iaea.org/public/download-endf/TENDL-2023",
        "info": "2023 release of the TENDL library distributed by the Paul Scherrer Institute (Switzerland).",
        "homepage": "https://tendl.web.psi.ch/tendl_2023/tendl2023.html",
        "index": [
            "1 NSUB=0      Materials:2825  Size:6Gb     Zipped:2Gb      30.MeV-200.MeV   [G]     Photo-Nuclear Data",
            "2 NSUB=10     Materials:2850  Size:12Gb    Zipped:3Gb      20.MeV-200.MeV   [N]     Incident-Neutron Data",
            "3 NSUB=10010  Materials:2855  Size:8Gb     Zipped:2Gb      2.MeV-200.MeV    [P]     Incident-Proton Data",
            "4 NSUB=10020  Materials:2850  Size:10Gb    Zipped:3Gb      1.MeV-200.MeV    [D]     Incident-Deuteron Data",
            "5 NSUB=10030  Materials:2865  Size:11Gb    Zipped:3Gb      20.MeV-200.MeV   [T]     Incident-Triton Data",
            "6 NSUB=20030  Materials:2821  Size:10Gb    Zipped:3Gb      20.MeV-200.MeV   [HE3]   Incident-He3 data",
            "7 NSUB=20040  Materials:2835  Size:6Gb     Zipped:2Gb      200.MeV          [HE4]   Incident-Alpha data",
            "",
            "Total: Materials:19901 Size:60Gb    Zipped:15Gb",
        ],
    },
    "cendl32": {
        "fancyname": "CENDL-3.2",
        "sublibraries": ["n"],
        "source": r"https://www-nds.iaea.org/public/download-endf/CENDL-3.2",
        "info": """Version 3.2 of the Chinese Evaluated Nuclear Data Library (CENDL)"""
        """ distributed by the China Nuclear Data Center.""",
        "homepage": "https://en.cnnc.com.cn/2020-06/17/c_501119.htm",
        "index": [
            "1 NSUB=10     Materials:272   Size:388Mb   Zipped:109Mb    20.MeV-50.MeV    [N]     Incident-Neutron Data",
            "",
            "Total: Materials:272   Size:388Mb   Zipped:109Mb",
        ],
    },
    "cendl31": {
        "fancyname": "CENDL-3.1",
        "sublibraries": ["n"],
        "source": r"https://www-nds.iaea.org/public/download-endf/CENDL-3.1",
        "info": """Version 3.1 of the Chinese Evaluated Nuclear Data Library (CENDL)"""
        """ distributed by the China Nuclear Data Center.""",
        "homepage": "https://en.cnnc.com.cn/2020-06/17/c_501119.htm",
        "index": [
            "1 NSUB=10     Materials:240   Size:156Mb   Zipped:36Mb     [N]     Incident-Neutron Data",
            "",
            "Total: Materials:240   Size:156Mb   Zipped:36Mb",
        ],
    },
}

ENDF6_SIZES = {
    "cendl31": {"n": 162612408},
    "endfb71": {
        "decay": 69003902,
        "sfpy": 1236314,
        "he3": 539478,
        "nfpy": 7148924,
        "tsl": 32976874,
        "std": 906346,
        "n": 849692528,
        "g": 232130930,
        "t": 680682,
        "ard": 8854278,
        "d": 407294,
        "photo": 23487096,
        "e": 24872486,
        "p": 55340816,
    },
    "jeff33": {
        "decay": 46349680,
        "sfpy": 379988,
        "nfpy": 4428656,
        "tsl": 156835655,
        "n": 1886369984,
    },
    "jeff311": {
        "decay": 43047048,
        "sfpy": 383678,
        "nfpy": 4348214,
        "tsl": 27657944,
        "n": 257227358,
        "p": 21205774,
    },
    "endfb8": {
        "decay": 68939532,
        "sfpy": 1236314,
        "he3": 829430,
        "he4": 8938,
        "nfpy": 7148924,
        "tsl": 240361270,
        "std": 67609410,
        "n": 1306128028,
        "g": 232131258,
        "t": 752678,
        "ard": 8919878,
        "d": 423448,
        "photo": 91438938,
        "e": 27489024,
        "p": 55582798,
    },
    "cendl32": {"n": 406146231},
}

OPENMC_LIBS: Dict[str, Dict[str, Dict[str, str | int]]] = {
    "official": {
        "endfb71": {
            "fancyname": "ENDF-B/VII.1",
            "source": "https://anl.box.com/shared/static/9igk353zpy8fn9ttvtrqgzvw1vtejoz6.xz",
            "tarname": "9igk353zpy8fn9ttvtrqgzvw1vtejoz6.xz",
            "extractedname": "endfb-vii.1-hdf5",
            "info": "Official OpenMC library based on ENDF-B/VII.1",
            "homepage": "https://openmc.org/official-data-libraries/",
            "size": 6242522633,
        },
        "endfb8": {
            "fancyname": "ENDF-B/VIII.0",
            "source": "https://anl.box.com/shared/static/uhbxlrx7hvxqw27psymfbhi7bx7s6u6a.xz",
            "tarname": "uhbxlrx7hvxqw27psymfbhi7bx7s6u6a.xz",
            "extractedname": "endfb-viii.0-hdf5",
            "info": "Official OpenMC library based on ENDF-B/VIII.0",
            "homepage": "https://openmc.org/official-data-libraries/",
            "size": 13679001942,
        },
        "jeff33": {
            "fancyname": "JEFF-3.3",
            "source": "https://anl.box.com/shared/static/4jwkvrr9pxlruuihcrgti75zde6g7bum.xz",
            "tarname": "4jwkvrr9pxlruuihcrgti75zde6g7bum.xz",
            "extractedname": "jeff-3.3-hdf5",
            "info": "Official OpenMC library based on JEFF-3.3",
            "homepage": "https://openmc.org/official-data-libraries/",
            "size": 9479124726,
        },
    },
    "lanl": {
        "endfb70": {
            "fancyname": "ENDF-B/VII.0",
            "source": "https://anl.box.com/shared/static/t25g7g6v0emygu50lr2ych1cf6o7454b.xz",
            "tarname": "t25g7g6v0emygu50lr2ych1cf6o7454b.xz",
            "extractedname": "mcnp_endfb70",
            "info": "ENDF-B/VII.0 based library converted from ACE files distributed with MCNP5/6",
            "homepage": "https://openmc.org/lanl-data-libraries/",
            "size": 2241962318,
        },
        "endfb71": {
            "fancyname": "ENDF-B/VII.1",
            "source": "https://anl.box.com/shared/static/d359skd2w6wrm86om2997a1bxgigc8pu.xz",
            "tarname": "d359skd2w6wrm86om2997a1bxgigc8pu.xz",
            "extractedname": "mcnp_endfb71",
            "info": "ENDF-B/VII.1 based library converted from ACE files distributed with MCNP5/6",
            "homepage": "https://openmc.org/lanl-data-libraries/",
            "size": 7186352635,
        },
        "endfb8": {
            "fancyname": "ENDF-B.VIII.0",
            "source": "https://anl.box.com/shared/static/nd7p4jherolkx4b1rfaw5uqp58nxtstr.xz",
            "tarname": "nd7p4jherolkx4b1rfaw5uqp58nxtstr.xz",
            "extractedname": "lib80x_hdf5",
            "info": "ENDF-B/VIII.0 based library converted from ACE files distributed by Los Alamos National lab (LANL)",
            "homepage": "https://openmc.org/lanl-data-libraries/",
            "size": 15774715647,
        },
    },
}

OPENMC_CHAINS = {
    "endfb71/thermal": {
        "url": "https://anl.box.com/shared/static/os1u896bwsbopurpgas72bi6aij2zzdc.xml",
        "size": 27657141,
        "info": "A chain based on the ENDF-B/VII.1 evaluation with thermal capture branching ratios",
    },
    "endfb71/fast": {
        "url": "https://anl.box.com/shared/static/9058zje1gm0ekd93hja542su50pccvj0.xml",
        "size": 27657870,
        "info": "A chain based on the ENDF-B/VII.1 evaluation with fast capture branching ratios",
    },
    "endfb8/thermal": {
        "url": "https://anl.box.com/shared/static/nyezmyuofd4eqt6wzd626lqth7wvpprr.xml",
        "size": 27526672,
        "info": "A chain based on the ENDF-B/VIII.0 evaluation with thermal capture branching ratios",
    },
    "endfb8/fast": {
        "url": "https://anl.box.com/shared/static/x3kp739hr5upmeqpbwx9zk9ep04fnmtg.xml",
        "size": 27527401,
        "info": "A chain based on the ENDF-B/VIII.0 evaluation with thermal capture branching ratios",
    },
    "casl/thermal": {
        "url": "https://anl.box.com/shared/static/3nvnasacm2b56716oh5hyndxdyauh5gs.xml",
        "size": 981527,
        "info": "A simplified chain as described by https://doi.org/10.2172/1256820 with thermal capture branching ratios",
    },
    "casl/fast": {
        "url": "https://anl.box.com/shared/static/9fqbq87j0tx4m6vfl06pl4ccc0hwamg9.xml",
        "size": 981581,
        "info": "A simplified chain as described by https://doi.org/10.2172/1256820 with fast capture branching ratios",
    },
}

SUBLIBRARIES: Dict[str, str] = {
    "n": "Incident-Neutron Data",
    "decay": "Radioactive Decay Data",
    "nfpy": "Neutron-Induced Fission Product Yields",
    "sfpy": "Spontaneous Fission Product Yields",
    "tsl": "Thermal Neutron Scattering Data",
    "ard": "Atomic Relaxation Data",
    "photo": "Photo-Atomic Interaction Data",
    "g": "Photo-Nuclear Data",
    "e": "Electro-Atomic Interaction Data",
    "p": "Incident-Proton Data",
    "d": "Incident-Deuteron Data",
    "t": "Incident-Tritium Data",
    "he4": "Incident-He4 Data",
    "he3": "Incident-He3 Data",
}

NSUB_IDS = {
    0: "g",
    3: "photo",
    4: "decay",
    5: "sfpy",
    6: "ard",
    10: "n",
    11: "nfpy",
    12: "tsl",
    10010: "p",
    10020: "d",
    10030: "t",
    20030: "he3",
    20040: "he4",
}

SUBLIBRARIES_SHORTLIST: List[str] = [
    "n",
    "decay",
    "nfpy",
    "sfpy",
    "tsl",
    "ard",
    "photo",
]

ATOMIC_SYMBOL: Dict[int | str, int | str] = {
    0: "n",
    1: "H",
    2: "He",
    3: "Li",
    4: "Be",
    5: "B",
    6: "C",
    7: "N",
    8: "O",
    9: "F",
    10: "Ne",
    11: "Na",
    12: "Mg",
    13: "Al",
    14: "Si",
    15: "P",
    16: "S",
    17: "Cl",
    18: "Ar",
    19: "K",
    20: "Ca",
    21: "Sc",
    22: "Ti",
    23: "V",
    24: "Cr",
    25: "Mn",
    26: "Fe",
    27: "Co",
    28: "Ni",
    29: "Cu",
    30: "Zn",
    31: "Ga",
    32: "Ge",
    33: "As",
    34: "Se",
    35: "Br",
    36: "Kr",
    37: "Rb",
    38: "Sr",
    39: "Y",
    40: "Zr",
    41: "Nb",
    42: "Mo",
    43: "Tc",
    44: "Ru",
    45: "Rh",
    46: "Pd",
    47: "Ag",
    48: "Cd",
    49: "In",
    50: "Sn",
    51: "Sb",
    52: "Te",
    53: "I",
    54: "Xe",
    55: "Cs",
    56: "Ba",
    57: "La",
    58: "Ce",
    59: "Pr",
    60: "Nd",
    61: "Pm",
    62: "Sm",
    63: "Eu",
    64: "Gd",
    65: "Tb",
    66: "Dy",
    67: "Ho",
    68: "Er",
    69: "Tm",
    70: "Yb",
    71: "Lu",
    72: "Hf",
    73: "Ta",
    74: "W",
    75: "Re",
    76: "Os",
    77: "Ir",
    78: "Pt",
    79: "Au",
    80: "Hg",
    81: "Tl",
    82: "Pb",
    83: "Bi",
    84: "Po",
    85: "At",
    86: "Rn",
    87: "Fr",
    88: "Ra",
    89: "Ac",
    90: "Th",
    91: "Pa",
    92: "U",
    93: "Np",
    94: "Pu",
    95: "Am",
    96: "Cm",
    97: "Bk",
    98: "Cf",
    99: "Es",
    100: "Fm",
    101: "Md",
    102: "No",
    103: "Lr",
    104: "Rf",
    105: "Db",
    106: "Sg",
    107: "Bh",
    108: "Hs",
    109: "Mt",
    110: "Ds",
    111: "Rg",
    112: "Cn",
    113: "Nh",
    114: "Fl",
    115: "Mc",
    116: "Lv",
    117: "Ts",
    118: "Og",
}
ATOMIC_SYMBOL |= {v: k for k, v in ATOMIC_SYMBOL.items()}

META_SYMBOL: Dict[str | int, int | str] = {"": 0, "M": 1, "N": 2, "O": 3}
META_SYMBOL |= {v: k for k, v in META_SYMBOL.items()}

TSL_NEUTRON: Dict[str, Dict[str, str]] = {
    "test": {
        "tsl_0056_26-Fe-56.endf6": "Fe56",
    },
    "jeff311": {
        "tsl_0001_H(H2O).endf6": "H1",
        "tsl_0007_H(ZrH).endf6": "H1",
        "tsl_0008_H(CaH2).endf6": "H1",
        "tsl_0011_D(D2O).endf6": "H2",
        "tsl_0026_4-Be.endf6": "Be9",
        "tsl_0031_Graphite.endf6": "C0",
        "tsl_0037_H(CH2).endf6": "H1",
        "tsl_0052_24-Mg.endf6": "Mg24",
        "tsl_0059_Ca(CaH2).endf6": "Ca40",
    },
    "jeff33": {
        "tsl_0001_H(H2O).endf6": "H1",
        "tsl_0002_para-H.endf6": "H1",
        "tsl_0003_ortho-H.endf6": "H1",
        "tsl_0007_H(ZrH).endf6": "H1",
        "tsl_0008_H(Cah2).endf6": "H1",
        "tsl_0010_H(ICE).endf6": "H1",
        "tsl_0011_D(D2O).endf6": "H2",
        "tsl_0012_para-D.endf6": "H2",
        "tsl_0013_ortho-D.endf6": "H2",
        "tsl_0026_4-Be.endf6": "Be9",
        "tsl_0031_Graphite.endf6": "C0",
        "tsl_0037_H(CH2).endf6": "H1",
        "tsl_0038_Mesi-PhII.endf6": "H1",
        "tsl_0042_Tolue-PhII.endf6": "H1",
        "tsl_0048_O(Al2O3).endf6": "O16",
        "tsl_0051_O(D2O).endf6": "O16",
        "tsl_0052_24-Mg.endf6": "Mg24",
        "tsl_0059_Si.endf6": "Si28",
        "tsl_0060_Al(Al2O3).endf6": "Al27",
        "tsl_0061_Ca(CaH2).endf6": "Ca40",
    },
    "endfb71": {
        "tsl_0001_H(H2O).endf6": "H1",
        "tsl_0002_para-H.endf6": "H1",
        "tsl_0003_ortho-H.endf6": "H1",
        "tsl_0007_H(ZrH).endf6": "H1",
        "tsl_0011_D(D2O).endf6": "H2",
        "tsl_0012_para-d.endf6": "H2",
        "tsl_0013_ortho-d.endf6": "H2",
        "tsl_0026_Bemetal.endf6": "Be9",
        "tsl_0027_Be(BeO).endf6": "Be9",
        "tsl_0031_graphite.endf6": "C0",
        "tsl_0033_l-ch4.endf6": "H1",
        "tsl_0034_s-ch4.endf6": "H1",
        "tsl_0037_H(CH2).endf6": "H1",
        "tsl_0040_BENZINE.endf6": "H1",
        "tsl_0046_O(BeO).endf6": "O16",
        "tsl_0047_SIO2.endf6": "Si28",
        "tsl_0048_U(UO2).endf6": "U238",
        "tsl_0053_13-Al-27.endf6": "Al27",
        "tsl_0056_26-Fe-56.endf6": "Fe56",
        "tsl_0058_Zr(ZrH).endf6": "Zr90",
        "tsl_0075_O(UO2).endf6": "O16",
    },
    "endfb8": {
        "tsl_0001_H(H2O).endf6": "H1",
        "tsl_0002_para-H.endf6": "H1",
        "tsl_0003_ortho-H.endf6": "H1",
        "tsl_0005_H(YH2).endf6": "H1",
        "tsl_0007_H(ZrH).endf6": "H1",
        "tsl_0010_H(ice-Ih).endf6": "H1",
        "tsl_0011_D(D2O).endf6": "H2",
        "tsl_0012_para-d.endf6": "H2",
        "tsl_0013_ortho-d.endf6": "H2",
        "tsl_0026_Be-metal.endf6": "Be9",
        "tsl_0027_Be(BeO).endf6": "Be9",
        "tsl_0030_Graphite.endf6": "C12",
        "tsl_0031_10PGraphit.endf6": "C12",
        "tsl_0032_30PGraphit.endf6": "C12",
        "tsl_0033_l-ch4.endf6": "H1",
        "tsl_0034_s-ch4.endf6": "H1",
        "tsl_0037_H(CH2).endf6": "H1",
        "tsl_0039_H(Lucite).endf6": "H1",
        "tsl_0040_BENZINE.endf6": "H1",
        "tsl_0043_Si(3C-SiC).endf6": "Si28",
        "tsl_0044_C(3C-SiC).endf6": "C12",
        "tsl_0046_O(BeO).endf6": "O16",
        "tsl_0047_SiO2alpha.endf6": "Si28",
        "tsl_0048_U(UO2).endf6": "U238",
        "tsl_0049_SiO2beta.endf6": "Si28",
        "tsl_0050_O(ice-Ih).endf6": "O16",
        "tsl_0051_O(D2O).endf6": "O16",
        "tsl_0053_13-Al-27.endf6": "Al27",
        "tsl_0055_Y(YH2).endf6": "Y89",
        "tsl_0056_26-Fe-56.endf6": "Fe56",
        "tsl_0058_Zr(ZrH).endf6": "Zr90",
        "tsl_0071_N(UN)L.endf6": "N14",
        "tsl_0072_U(UN)L.endf6": "U238",
        "tsl_0075_O(UO2).endf6": "O16",
    },
    "jendl5": {
        "tsl_013-Al-27_0053.endf6": "Al27",
        "tsl_026-Fe-56_0056.endf6": "Fe56",
        "tsl_10PGraphit_0031.endf6": "C12",
        "tsl_30PGraphit_0032.endf6": "C12",
        "tsl_Be(BeO)_0027.endf6": "Be9",
        "tsl_Be-metal_0026.endf6": "Be9",
        "tsl_C(3C-SiC)_0044.endf6": "C12",
        "tsl_C(C19H16)_0644.endf6": "C12",
        "tsl_C(C19H16)_0645.endf6": "C12",
        "tsl_C(C2H6O)_0636.endf6": "C12",
        "tsl_C(C2H6O)_0637.endf6": "C12",
        "tsl_C(C6H6)_0640.endf6": "C12",
        "tsl_C(C6H6)_0641.endf6": "C12",
        "tsl_C(C7H8)_0635.endf6": "C12",
        "tsl_C(C7H8)_0642.endf6": "C12",
        "tsl_C(C9H12)_0632.endf6": "C12",
        "tsl_C(C9H12)_0638.endf6": "C12",
        "tsl_C(CH4)_0633.endf6": "C12",
        "tsl_C(CH4)_0634.endf6": "C12",
        "tsl_C(m-C8H10)_0639.endf6": "C12",
        "tsl_C(m-C8H10)_0643.endf6": "C12",
        "tsl_D(D2O)_0011.endf6": "H2",
        "tsl_Graphite_0030.endf6": "C12",
        "tsl_H(C19H16)_0614.endf6": "H1",
        "tsl_H(C19H16)_0615.endf6": "H1",
        "tsl_H(C2H6O)_0606.endf6": "H1",
        "tsl_H(C2H6O)_0607.endf6": "H1",
        "tsl_H(C6H6)_0040.endf6": "H1",
        "tsl_H(C6H6)_0611.endf6": "H1",
        "tsl_H(C7H8)_0042.endf6": "H1",
        "tsl_H(C7H8)_0605.endf6": "H1",
        "tsl_H(C9H12)_0038.endf6": "H1",
        "tsl_H(C9H12)_0602.endf6": "H1",
        "tsl_H(CH2)_0037.endf6": "H1",
        "tsl_H(CH4)_0033.endf6": "H1",
        "tsl_H(CH4)_0034.endf6": "H1",
        "tsl_H(H2O)_0001.endf6": "H1",
        "tsl_H(ice-Ih)_0010.endf6": "H1",
        "tsl_H(Lucite)_0039.endf6": "H1",
        "tsl_H(m-C8H10)_0609.endf6": "H1",
        "tsl_H(m-C8H10)_0613.endf6": "H1",
        "tsl_H(YH2)_0005.endf6": "H1",
        "tsl_H(ZrH)_0007.endf6": "H1",
        "tsl_N(UN)L_0071.endf6": "N14",
        "tsl_O(BeO)_0046.endf6": "O16",
        "tsl_O(C2H6O)_0666.endf6": "O16",
        "tsl_O(C2H6O)_0667.endf6": "O16",
        "tsl_O(D2O)_0051.endf6": "O16",
        "tsl_O(H2O)_0661.endf6": "O16",
        "tsl_O(ice-Ih)_0050.endf6": "O16",
        "tsl_ortho-D_0013.endf6": "H2",
        "tsl_ortho-H_0003.endf6": "H1",
        "tsl_O(UO2)_0075.endf6": "O16",
        "tsl_para-D_0012.endf6": "H2",
        "tsl_para-H_0002.endf6": "H1",
        "tsl_Si(3C-SiC)_0043.endf6": "Si28",
        "tsl_SiO2alpha_0047.endf6": "Si28",
        "tsl_SiO2beta_0049.endf6": "Si28",
        "tsl_U(UN)L_0072.endf6": "U238",
        "tsl_U(UO2)_0048.endf6": "U238",
        "tsl_Y(YH2)_0055.endf6": "Y89",
        "tsl_Zr(ZrH)_0058.endf6": "Zr90",
    },
}

TAPE_SHA1 = {
    "cendl31": CENDL31_SHA1,
    "cendl32": CENDL32_SHA1,
    "endfb71": ENDFB71_SHA1,
    "endfb8": ENDFB8_SHA1,
    "jeff311": JEFF311_SHA1,
    "jeff33": JEFF33_SHA1,
    "jendl5": JENDL5_SHA1,
    "tendl19": TENDL19_SHA1,
    "tendl23": TENDL23_SHA1,
    "test": TEST_SHA1,
}
